package Clustericious::Command::generate::mbd_app;

use strict;
use warnings;
use Mojo::Base 'Clustericious::Command';
use File::Find;
use File::ShareDir 'dist_dir';
use File::Basename qw( basename );

# ABSTRACT: Clustericious command to generate a new Clustericious M::B::D application
our $VERSION = '1.10_04'; # TRIAL VERSION



has description => <<'EOF';
Generate Clustericious app based on Module::Build::Database.
EOF

has usage => <<"EOF";
usage: $0 generate mbd_app [NAME]
EOF

sub run
{
  say STDERR "ERROR: this command has been removed";
  exit 2;
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Clustericious::Command::generate::mbd_app - Clustericious command to generate a new Clustericious M::B::D application

=head1 VERSION

version 1.10_04

=head1 DESCRIPTION

This command has been removed.

=head1 SEE ALSO

L<Clustericious>

=head1 AUTHOR

Original author: Brian Duggan

Current maintainer: Graham Ollis E<lt>plicease@cpan.orgE<gt>

Contributors:

Curt Tilmes

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by NASA GSFC.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
